# cython: linetrace=True
# distutils: define_macros=CYTHON_TRACE_NOGIL=1
'''Cython implementation of resampler'''

import cython
import numpy as np
cimport numpy as cnp

from cython import floating
from cython cimport floating


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef void resample_f(floating[:, :] x, floating[:, :] y, 
                      double sample_ratio,
                      double[:] interp_win, double[:] interp_delta, int num_table) nogil:
    
    cdef:
        double scale = min(1.0, sample_ratio)
        double time_increment = 1./sample_ratio
        int index_step = int(scale * num_table)
        double time_register = 0.0
    
        int n = 0
        double frac = 0.0
        double index_frac = 0.0
        int offset = 0
        double eta = 0.0
        double weight = 0.0
    
        int nwin = interp_win.shape[0]
        int n_orig = x.shape[0]
        int n_out = y.shape[0]
        int n_channels = y.shape[1]

        int t, i, j

    for t in range(n_out):
        # Grab the top bits as an index to the input buffer
        n = int(time_register)
        
        # Grab the fractional component of the time index
        frac = scale * (time_register - n)
        
        # Offset into the filter
        index_frac = frac * num_table
        offset = int(index_frac)
        
        # Interpolation factor
        eta = index_frac - offset
        
        # Compute the left wing of the filter response
        for i in range(min(n + 1, (nwin - offset) // index_step)):

            weight = (interp_win[offset + i * index_step] + eta * interp_delta[offset + i * index_step])
            for j in range(n_channels):
                y[t, j] += weight * x[n - i, j]
        
        # Invert P
        frac = scale - frac

        # Offset into the filter
        index_frac = frac * num_table
        offset = int(index_frac)
        
        # Interpolation factor
        eta = index_frac - offset
        
        # Compute the right wing of the filter response
        for i in range(min(n_orig - i - n + 1, (nwin - offset)//index_step)):
            weight = (interp_win[offset + i * index_step] + eta * interp_delta[offset + i * index_step])
            for j in range(n_channels):
                y[t, j] += weight * x[n + i + 1, j]

        # Increment the time register
        time_register += time_increment
    pass




from setuptools import setup
from setuptools.extension import Extension

import numpy as np

import imp

version = imp.load_source('resampy.version', 'resampy/version.py')

setup(
    author="Brian McFee",
    author_email="brian.mcfee@nyu.edu",
    name='resampy',
    version=version.version,
    url='https://github.com/bmcfee/resampy',
    download_url='https://github.com/bmcfee/resampy/releases',
    description='Efficient signal resampling',
    license='ISC',
    ext_modules=[Extension("resampy.interp",
                           ["resampy/interp.pyx"],
                           include_dirs=[np.get_include()] )],
    packages=['resampy'],
    package_data={'resampy': ['data/*']},
    install_requires=[
        'numpy>=1.10',
        'scipy>=0.13',
        'six>=1.3',
        'Cython>=0.21'],
    extras_require={
        'docs': [
            'sphinx!=1.3.1',  # autodoc was broken in 1.3.1
            'numpydoc',
        ],
        'tests': [
            'pytest',
            'pytest-cov',
        ],
    },
    classifiers=[
        "License :: OSI Approved :: ISC License (ISCL)",
        "Programming Language :: Python",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Multimedia :: Sound/Audio :: Analysis",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ],
) 
